/*
 * Copyright (C) 2006 Romain HERAULT
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "pz.h"


static PzModule *dice_module;
static const char *dice_n_str[] = {"1","2","3","4","5","6","7","8","9","10","11","12",0};
static const char *dice_type_str[] = {"2","3","4","6","8","10","12","20","100",0};
static const char *dice_style_str[] = {"Figures","Points",0};
static int dice_type_n[]={2,3,4,6,8,10,12,20,100};
static int dice_type_l[]={1,1,1,1,1,1,2,2,2};
static int dice_row[3][12]=
 	{{1,2,1,2,2,3,2,3,3,3,4,4},
 	 {0,0,2,2,3,3,3,2,3,4,3,4},
 	 {0,0,0,0,0,0,2,3,3,3,4,4}};
static int dice_style[9][9]=
	{{0,0,0,0,1,0,0,0,0},
	 {0,0,1,0,0,0,1,0,0},
	 {0,0,1,0,1,0,1,0,0},
	 {1,0,1,0,0,0,1,0,1},
	 {1,0,1,0,1,0,1,0,1},
	 {1,0,1,1,0,1,1,0,1},
	 {1,0,1,1,1,1,1,0,1},
	 {1,1,1,1,0,1,1,1,1},
	 {1,1,1,1,1,1,1,1,1}};
static int dice_tab[12];
static char dice_title[64]="";

#define DAP_BG		(0)
#define DAP_BTOP	(1)
#define DAP_BBOT	(2)
#define DAP_BSIDE	(3)
#define DAP_BCORN	(4)
#define DAP_TEXT	(5)

static char *dice_appearance[6] =
	{	"box.default.bg",
		"box.default.border",
		"box.default.border",
		"box.default.border",
		"window.bg",
		"box.default.fg" };

static void dice_surface_draw(ttk_surface srf,int x, int y, int c)
{
	int x1,y1,x2,y2;
	x1=(x*2-c)/2;
	y1=(y*2-c)/2;
	x2=(x*2+c)/2;
	y2=(y*2+c)/2;
	if(ttk_screen->bpp == 16){
		/* background fill */
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BG]),
				x1, y1, x2, y2);
		/* border - this should be a proper roundrect */
			/* top */
		ttk_ap_hline(srf, ttk_ap_getx(dice_appearance[DAP_BTOP]),
				x1, x2 - 1, y1);
			/* bottom */
		ttk_ap_hline(srf, ttk_ap_getx(dice_appearance[DAP_BBOT]),
				x1, x2 - 1, y2 - 1);
			/* sides */
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BSIDE]),
				x1, y1, x1, y2);
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BSIDE]),
				x2 - 1, y1, x2 - 1, y2);
			/* corners */
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BCORN]),
				x1, y1, x1, y1);
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BCORN]),
				x2 - 1, y1, x2 - 1, y1);
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BCORN]),
				x1, y2 - 1, x1, y2 - 1);
		ttk_ap_fillrect(srf, ttk_ap_getx(dice_appearance[DAP_BCORN]),
				x2 - 1, y2 - 1, x2 - 1, y2 - 1);
	} else {
		ttk_fillrect(srf, x1, y1, x2, y2, ttk_makecol(WHITE));
		ttk_rect(srf, x1, y1, x2, y2, ttk_makecol(BLACK));
	}
}

static void dice_figure_draw(ttk_surface srf,int x, int y, int c, int num)
{
	int ch,cw,kern,i,numsize;
	char s[4];
	int  n[3];
	/* Making the string */
	numsize=dice_type_l[pz_menu_get_option("/Extras/Games/Dice/Type")];
	if(numsize>3)
		numsize=3;
	for(i=0;i<numsize;i++){
		n[numsize-i-1]=num%10;
		num-=n[numsize-i-1];
		num/=10;
	}
	for(i=0;i<numsize;i++)
		snprintf(s+i,2,"%d",n[i]);
	ch=(8*c)/10;
	cw=(ch*5)/9;
	kern=0;
	if(ttk_screen->bpp == 16){
		while( ((cw*numsize) > (c-12)) && (cw>3) )
		{
 			cw--;
 			ch=(cw*9)/5;
 		}
		pz_vector_string_center(srf,s,x,y,cw,ch,kern,
				ttk_ap_getx(dice_appearance[DAP_TEXT])->color);
	} else {
		while( ((cw*numsize) > (c-8)) && (cw>3) )
		{
 			cw--;
 			ch=(cw*9)/5;
 		}
		pz_vector_string_center(srf,s,x,y,cw,ch,kern,ttk_makecol(BLACK));
	}
}

static void dice_point_draw(ttk_surface srf,int x, int y, int c, int num)
{
	int i,j,p,r;
	ttk_color col;
	if(ttk_screen->bpp == 16){
		p=(c-3)/4;
		col=ttk_ap_getx(dice_appearance[DAP_TEXT])->color;
	} else {
		p=(c-1)/4;
		col=ttk_makecol(BLACK);
	}
	r=(p*7)/20;
	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			if(dice_style[num-1][i*3+j]==1)
				ttk_fillellipse(srf,x+(j-1)*p,y+(i-1)*p,r,r,col);
}

static void dice_draw(ttk_surface srf,int x, int y, int c, int num)
{
	dice_surface_draw(srf,x,y,c);
	if((dice_type_n[pz_menu_get_option("/Extras/Games/Dice/Type")]<10)&&
		(pz_menu_get_option("/Extras/Games/Dice/Style")== 1))
	{
		dice_point_draw(srf,x,y,c,num);
	}
	else
	{
		dice_figure_draw(srf,x,y,c,num);
	}
}

static void roll_draw(PzWidget *wid, ttk_surface srf)
{
	int n_dices,n_faces,n_size;
	int c,oc,nh,nw,marh,marw,lw,offh,offw,steph,stepw;
	int i,j,k,sum;
	
	ttk_ap_fillrect(srf, ttk_ap_getx("window.bg"), 0, 0,
		wid->w, wid->h);
	n_dices=pz_menu_get_option("/Extras/Games/Dice/Number")+1;
	n_faces=dice_type_n[pz_menu_get_option("/Extras/Games/Dice/Type")];
	for(sum=k=0;k<n_dices;k++){
		dice_tab[k]=1+rand()%n_faces;
		sum+=dice_tab[k];
	}
	snprintf(dice_title,64,_("Total: %d"),sum);
	marh=(5*wid->h)/100;
	marw=(5*wid->w)/100;
	nh=0;
	nw=0;
	for(i=0;i<3;i++){
		lw=dice_row[i][n_dices-1];
		if(lw==0)
			break;
		nh++;
		nw=nw>lw?nw:lw;
	}
	steph=(wid->h-marh)/nh;
	stepw=(wid->w-marw)/nw;
	oc=steph>stepw?stepw:steph;
	c=(oc*95)/100;
	offh=(marh+steph)/2;
	for(k=0,i=0;i<nh;i++){
		lw=dice_row[i][n_dices-1];
		offw=(marw+(1+nw-lw)*stepw)/2;
		for(j=0;j<lw;j++,k++)
			dice_draw(srf,offw+j*stepw,offh+i*steph,c,dice_tab[k]);
	}
}


static int roll_events(PzEvent *e)
{
	int ret = TTK_EV_CLICK;

	switch(e->type) {
	case PZ_EVENT_BUTTON_DOWN:
		switch (e->arg) {
			case PZ_BUTTON_MENU:
				pz_close_window(e->wid->win);
				break;
			case PZ_BUTTON_ACTION:
				roll_draw(e->wid,e->wid->win->srf);
				e->wid->win->dirty|=TTK_FILTHY;
				break;
			default:
				ret = TTK_EV_UNUSED;
		}
		break;
	default:
		ret = TTK_EV_UNUSED;
		break;
	}
	return ret;
}

static PzWindow *new_roll_win()
{
	PzWindow *ret;

	ret = pz_new_window(0, PZ_WINDOW_NORMAL);
	ret->title=dice_title;
	ret->titlefree=0;
	pz_add_widget(ret, roll_draw, roll_events);

	return pz_finish_window(ret);
}

static void init_dice()
{
	dice_module = pz_register_module("dice", NULL);
	pz_menu_add_stub("/Extras/Games/Dice");
	pz_menu_add_option("/Extras/Games/Dice/Number", dice_n_str);
	pz_menu_set_option("/Extras/Games/Dice/Number", 2);
	pz_menu_add_option("/Extras/Games/Dice/Type", dice_type_str);
	pz_menu_set_option("/Extras/Games/Dice/Type", 3);
	pz_menu_add_option("/Extras/Games/Dice/Style", dice_style_str);
	pz_menu_set_option("/Extras/Games/Dice/Style", 1);
	pz_menu_add_action("/Extras/Games/Dice/Roll", new_roll_win);
}

PZ_MOD_INIT(init_dice)


